# Fischertechnik TXT

Dieses Projekt dient dem vollständig automatischen Einrichten des Fischertechnik-TXT für die Entwicklung des RoboticStudio. Dazu wird ein SSH-Zugriff ohne Login auf den TXT eingerichtet, entsprechende Berechtigungen hinterlegt, Python installiert und der eigentliche Quellcode auf dem Server hinterlegt. Das Projekt gliedert sich im Wesentlichen in 3 Ordner:

- deploy: Hier sind die Skripte zum Deployment auf dem TXT untergebracht.
- flaskApp: Hier befindet sich die grafische Oberfläche, mit der der Swagger-Server auf dem TXT gesteuert werden kann.
- flaskConnnexion: Hier liegt der eigentliche Swagger-Server.

Üblicherweise wird beim ersten Einrichten des TXT einmal das komplette Deployment durchgeführt; nach einem Neustart lediglich der Task `mount`. Wird am Swagger-Server entwickelt, so kann mit `deploy_flask_app` der Python-Code der grafischen Oberfläche und des Swagger-Servers aktualisiert werden.

## Vorraussetzungen
Auf dem Rechner, von dem aus deployt werden soll, muss `python3` installiert sein. Zusätzlich wird die Python-Bibliothek `fabric` (https://github.com/fabric/fabric) benötigt. Darüber hinaus muss der TXT im selben WLAN sein und die Fischertechnik-CommunityEdition gebootet sein.

## Ausführen des Skriptes
Ist die Bibliothek `fabric` lokal installiert, steht der Command `fab` zum Ausführen der Tasks aus der sogenannten Fabfile zur Verfügung. Diese beinhaltet fünf verschiedene Tasks zum Deployment, die im folgenden erklärt werden. Es können alle Tasks mit dem Befehl `fab --list` angezeigt werden

### Komplettes Deployment durchführen: deploy
Der auszuführende Task lautet:
```bash
fab deploy -h '<ip>'
```
Dieser ist ein Sammeltask und führt die folgenden 4 Tasks nacheinander aus. Das Skript beinhaltet einige Print-Ausgaben und Anweisungen, sodass der Prozess des Einrichtens auf der lokalen Kommandozeile dokumentiert wird.

### SSH-Key auf den Server kopieren: copy_ssh_key
Der auszuführende Task lautet:
```bash
fab copy_ssh_key -h '<ip>'
```
Damit die Python-Skripte auf dem Controller ausgeführt werden können, muss der Public-SSH-Key des Rechners, von dem deployt werden soll, auf dem TXT in der Datei `.ssh/authorized_keys` liegen. Ist dies nicht der Fall werden je nach eigener SSH-Konfiguration unterschiedliche Fehler geworfen und das Deployment läuft **nicht** durch. Das Kopieren des SSH-Keys übernimmt das Skript automatisch; es lädt die Datei `.ssh/id_rsa.pub` und kopiert diese mittels SCP auf den Server in die entsprechende Datei. Hierzu ist zweimal das Erlauben des SSH-Logins über die Nutzeroberfläche des TXT notwendig.

### Prüfen der Mounts: mount
Der auszuführende Task lautet:
```bash
fab mount -h '<ip>'
```
Damit die Python Skripte korrekt ausgeführt werden können, müssen die Mount-Points richtig initialisiert werden. Das Skript prüft automatisch, ob die Mounts in der `/etc/mtab` eingetragen sind und wenn nicht versucht es diese hinzuzufügen. Dazu muss auf dem Display zweimal der Root-Zugriff bestätigt werden. Stellt das Skript darüber hinaus fest, dass die Mount-Points noch nicht einmal initialisiert wurden, erstellt es die Verzeichnisse unter `~/mounts` und kopiert die Dateien in diese. Eine Besonderheit stellt dabei die Datei `/usr/bin/sudo`, also der Sudo-Command, dar. Diese Datei muss den Eigentümer root und das setuid-Bit muss gesetzt sein. Um diesen Schritt kümmert sich das Skript ebenfalls automatisiert, jedoch ist hier zusätzlich zweimal der Root-Zugriff auf dem Display zu bestätigen.

### Prüfen von Pip und den benötigten Dependencies: install_python
Der auszuführende Task lautet:
```bash
fab install_python -h '<ip>'
```
Damit alle Anforderungen für den Flask-Server vorhanden sind, werden alle Dependencies auf dem TXT geprüft. Im ersten Schritt wird geprüft, ob pip installiert ist. Ist dies nicht der Fall, wird pip auf dem TXT installiert. Danach werden mittels des Commands `pip list --format=freeze` einmal alle installierten Bibliotheken ausgegeben und mit der lokalen Datei `flaskConnexion/requirements.txt` abgeglichen. Fehlen Bibliotheken, werden diese nachinstalliert. Dies bedeutet auch, dass nach einer Anpassung der lokalen Requirements die neuen Bibliotheken auch auf dem TXT installiert werden.

### Hochladen des Flask-Servers: deploy_flask_app
Der auszuführende Task lautet:
```bash
fab deploy_flask_app -h '<ip>'
```
Zuletzt muss noch der Quellcode auf den TXT geladen werden. Hierzu werden diw Order `flaskConnexion` (Flask-Server) und `flaskApp` (GUI-App) mithilfe von tar zu einem Archiv gepackt, dann auf den TXT hochgeladen und wieder entpackt. Die GUI-App wird derart angepasst, dass diese ausführbar ist. Zusätzlich wird die Oberfläche des TXT neu geladen, damit die neue App erscheint. Anschließend wird die IP in der `swagger.yml` auf die aktuelle IP des TXT gesetzt.

## Verwendung der Flask-App

Um die App zu Starten geht man auf dem TXT in den Ordner Demos und startet die Anwendung `Flask`. Die Anwendung läuft in zwei separaten Prozessen; einer für die grafische Oberfläche und ein zweiter für den Swagger-Server. Die grafische Oberfläche hat eine Status-Anzeige sowie jeweils einen Start- und Stop-Button. Mit dem Start Button wird der Swagger-Server gestartet; dies dauert ca. 25 Sekunden. Danach kann die grafische Oberfläche verlassen werden; der Swagger-Server läuft im Hintergrund weiter. Die Swagger-UI ist unter http://ip-of-my-txt:8080/v1/ui erreichbar; die API selbst unter steht unter http://ip-of-my-txt:8080/v1 zur Verfügung. Mit dem Stop-Button wird der Swagger-Server beendet.