from deploy import colorized_output as co

class PythonEnvironment:
    def __init__(self, connection):
        self.connection = connection

    def add_ftc_path(self):
        co.print_info('Add /opt/ftc to PYTHONPATH')
        self.connection.run('echo "export PYTHONPATH=/opt/ftc" > /home/ftc/.profile', warn=True)
        co.print_success('--> Important for ui applications')

    def check_pip(self):
        co.print_info("Checking pip-installation")
        if self.connection.run('ls /usr/bin | grep pip3.7', warn=True):
            co.print_success('--> Already installed')
        else:
            co.print_warning('--> Not installed. Starting installation of pip')
            self.install_pip('~/tmp')
            co.print_success('--> Successfully installed pip')

    def install_pip(self, tmp_dir):
        self.connection.run("mkdir -p {0}".format(tmp_dir))
        self.connection.run("wget https://bootstrap.pypa.io/get-pip.py -O/home/ftc/tmp/get-pip.py".format(tmp_dir))
        self.connection.run("python {0}/get-pip.py".format(tmp_dir))
        self.connection.run("rm -rf {0}".format(tmp_dir))

    def check_pip_libraries(self):
        co.print_info('Checking pip libraries')
        requirements = self.get_requirements()
        pip_list = self.get_pip_list()
        for requirement in requirements:
            search = requirement.strip().replace(' ', '')
            self.check_pip_library(pip_list, search)

    def check_pip_library(self, dependencies, dependency):
        co.print_info("Checking {0}:".format(dependency))
        if dependency in dependencies:
            co.print_success("--> Already installed")
        else:
            co.print_warning("--> Not installed. Starting installation")
            if dependency.find('ft-') == -1 and dependency.find('ftrobopy') == -1 and dependency.find('txtapi') == -1:
                self.connection.run("pip install {0}".format(dependency))
            else:
                co.print_info("Downloading from custom repository")
                co.print_info("Enter credentials if needed")
                self.connection.run("pip install --index-url https://update.fischertechnik-cloud.com/repository/ft-txt-lib/simple/ --no-deps {0}".format(dependency))
            co.print_success("--> Successfully installed")

    def get_pip_list(self):
        return self.connection.run('pip list --format=freeze').stdout.strip().split()

    def get_requirements(self):
        fd = open('../txtapi/requirements.txt', 'r')
        return fd.readlines()
