"""
Enable rpdb to be started with command line.
"""
import rpdb

EOT = "debugger-message-end\0"


class RpdbHelper(rpdb.Rpdb):

    # overwrite functions without shutdown() so that stdout will not be closed.

    def do_continue(self, arg):
        import pdb # import inside function or else 'pdb' will be undefined
        try:
            return pdb.Pdb.do_continue(self, arg)
        finally:
            pass

    do_c = do_cont = do_continue

    def do_quit(self, arg):
        import pdb
        try:
            return pdb.Pdb.do_quit(self, arg)
        finally:
            pass

    do_q = do_exit = do_quit

    def do_EOF(self, arg):
        import pdb
        try:
            return pdb.Pdb.do_EOF(self, arg)
        finally:
            pass


def main():
    import traceback
    import sys
    from txtapi.utility.debugger.rpdb_helper import EOT
    from txtapi.utility.watch_process import WatchProcess

    port = sys.argv[1]
    mainpyfile = sys.argv[2]
    sys.argv = sys.argv[2:]
    rpdbh = RpdbHelper(port=int(port))
    rpdbh.prompt = EOT

    while True:
        try:
            rpdbh._runscript(mainpyfile)
            if rpdbh._user_requested_quit:
                break
        except SystemExit:
            sys.__stdout__.write("The program exited via sys.exit(). Exit status: {}\n".format(sys.exc_info()[1]))
        except SyntaxError:
            traceback.print_exc()
            sys.exit(1)
        except:
            sys.__stdout__.write("Uncaught exception.\n")
            traceback.print_exc()
            break
        sys.__stdout__.write("Debugger finished.\n")
        WatchProcess.get_instance().send_finished()
        break


if __name__ == '__main__':
    main()
